# fractal tree
# contributed by Gerhard Schmid and André Wobst

from pyx import *

# base tree length
l = 5

# base transformations for the left, center, and right part of the tree
ltrafo = trafo.rotate(65).scaled(0.4).translated(0, l * 2.0 / 3.0)
ctrafo = trafo.rotate(-4).scaled(0.75).translated(0, l)
rtrafo = trafo.mirror(90).rotated(-65).scaled(0.35).translated(0, l)

def tree(depth):
    "return transformations for a recursive tree of given depth"
    r = [trafo.rotate(5)]
    if depth > 0:
        subtree = tree(depth - 1)
        r.extend([t*ltrafo for t in subtree])
        r.extend([t*ctrafo for t in subtree])
        r.extend([t*rtrafo for t in subtree])
    return r

c = canvas.canvas()
for t in tree(7):
    # apply the transformation to a "sub"-canvas and insert it into the "main" canvas
    c.insert(canvas.canvas([t])).stroke(path.line(0, 0, 0, l))
    # note that there is a difference when only transforming the line as in:
    # c.stroke(path.line(0, 0, 0, l), [t])
    # The difference is, that the linewidth would not be scaled down.
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import *
from pyx.graph.axis.tick import tick, rational
from pyx.graph.axis.parter import lin, log, preexp


class ParterTestCase(unittest.TestCase):

    def PartEqual(self, part1, part2):
        self.assertEqual(len(part1), len(part2))
        for tick1, tick2 in zip(part1, part2):
            self.assertEqual(tick1, tick2)
            self.assertEqual(tick1.ticklevel, tick2.ticklevel)
            self.assertEqual(tick1.labellevel, tick2.labellevel)
            self.assertEqual(tick1.label, tick2.label)

    def testLinParter(self):
        self.PartEqual(lin(tickdists=["10"], labeldists=[]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, None), tick((10, 1), 0, None)])
        self.PartEqual(lin(tickdists=["1", "0.5"], labeldists=[]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, None), tick((1, 2), 1, None), tick((1, 1), 0, None)])
        self.PartEqual(lin(tickdists=["1", "0.5", "0.25"], labeldists=[]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, None), tick((1, 4), 2, None), tick((1, 2), 1, None), tick((3, 4), 2, None), tick((1, 1), 0, None)])
        self.PartEqual(lin(tickdists=["1", "0.5"], labeldists=[], extendtick=None).partfunctions(0, 1.5, 1, 1)[0](),
                       [tick((0, 1), 0, None), tick((1, 2), 1, None), tick((1, 1), 0, None), tick((3, 2), 1, None)])
        self.PartEqual(lin(tickdists=["1", "0.5"], labeldists=[], extendtick=1).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), 0, None), tick((1, 2), 1, None), tick((1, 1), 0, None), tick((3, 2), 1, None)])
        self.PartEqual(lin(tickdists=["1", "0.5"], labeldists=[], extendtick=0).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), 0, None), tick((1, 2), 1, None), tick((1, 1), 0, None), tick((3, 2), 1, None), tick((2, 1), 0, None)])
        self.PartEqual(lin(labeldists=["1"], tickdists=[]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), None, 0), tick((1, 1), None, 0)])
        self.PartEqual(lin(labeldists=["1", "0.5"], tickdists=[]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), None, 0), tick((1, 2), None, 1), tick((1, 1), None, 0)])
        self.PartEqual(lin(labeldists=["1", "0.5", "0.25"], tickdists=[]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), None, 0), tick((1, 4), None, 2), tick((1, 2), None, 1), tick((3, 4), None, 2), tick((1, 1), None, 0)])
        self.PartEqual(lin(labeldists=["1", "0.5"], tickdists=[], extendlabel=None).partfunctions(0, 1.5, 1, 1)[0](),
                       [tick((0, 1), None, 0), tick((1, 2), None, 1), tick((1, 1), None, 0), tick((3, 2), None, 1)])
        self.PartEqual(lin(labeldists=["1", "0.5"], tickdists=[], extendlabel=1).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), None, 0), tick((1, 2), None, 1), tick((1, 1), None, 0), tick((3, 2), None, 1)])
        self.PartEqual(lin(labeldists=["1", "0.5"], tickdists=[], extendlabel=0).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), None, 0), tick((1, 2), None, 1), tick((1, 1), None, 0), tick((3, 2), None, 1), tick((2, 1), None, 0)])
        self.PartEqual(lin(tickdists=["1"]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 1), 0, 0)])
        self.PartEqual(lin(tickdists=["1", "0.5"]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), 1, None), tick((1, 1), 0, 0)])
        self.PartEqual(lin(tickdists=["1", "0.5", "0.25"]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 4), 2, None), tick((1, 2), 1, None), tick((3, 4), 2, None), tick((1, 1), 0, 0)])
        self.PartEqual(lin(tickdists=["1", "0.5"], extendtick=None).partfunctions(0, 1.5, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), 1, None), tick((1, 1), 0, 0), tick((3, 2), 1, None)])
        self.PartEqual(lin(tickdists=["1", "0.5"], extendtick=1).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), 1, None), tick((1, 1), 0, 0), tick((3, 2), 1, None)])
        self.PartEqual(lin(tickdists=["1", "0.5"], extendtick=0).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), 1, None), tick((1, 1), 0, 0), tick((3, 2), 1, None), tick((2, 1), 0, 0)])
        self.PartEqual(lin(labeldists=["1"]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 1), 0, 0)])
        self.PartEqual(lin(labeldists=["1", "0.5"]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), None, 1), tick((1, 1), 0, 0)])
        self.PartEqual(lin(labeldists=["1", "0.5", "0.25"]).partfunctions(0, 1, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 4), None, 2), tick((1, 2), None, 1), tick((3, 4), None, 2), tick((1, 1), 0, 0)])
        self.PartEqual(lin(labeldists=["1", "0.5"], extendtick=None).partfunctions(0, 1.5, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), None, 1), tick((1, 1), 0, 0), tick((3, 2), None, 1)])
        self.PartEqual(lin(labeldists=["1", "0.5"], extendtick=None, extendlabel=1).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), None, 1), tick((1, 1), 0, 0), tick((3, 2), None, 1)])
        self.PartEqual(lin(labeldists=["1", "0.5"], extendtick=None, extendlabel=0).partfunctions(0, 1.2, 1, 1)[0](),
                       [tick((0, 1), 0, 0), tick((1, 2), None, 1), tick((1, 1), 0, 0), tick((3, 2), None, 1), tick((2, 1), 0, 0)])

    def testLogParter(self):
        self.PartEqual(log(tickpreexps=[log.pre1exp], labelpreexps=[]).partfunctions(1, 10, 1, 1)[0](),
                       [tick((1, 1), 0, None), tick((10, 1), 0, None)])
        self.PartEqual(log(tickpreexps=[log.pre1exp], labelpreexps=[]).partfunctions(1, 100, 1, 1)[0](),
                       [tick((1, 1), 0, None), tick((10, 1), 0, None), tick((100, 1), 0, None)])
        self.PartEqual(log(tickpreexps=[preexp([rational((1, 1))], 100)], labelpreexps=[]).partfunctions(1, 100, 1, 1)[0](),
                       [tick((1, 1), 0, None), tick((100, 1), 0, None)])
        self.PartEqual(log(tickpreexps=[log.pre1to9exp], labelpreexps=[]).partfunctions(1, 10, 1, 1)[0](),
                       [tick((1, 1), 0, None), tick((2, 1), 0, None), tick((3, 1), 0, None), tick((4, 1), 0, None), tick((5, 1), 0, None), tick((6, 1), 0, None), tick((7, 1), 0, None), tick((8, 1), 0, None), tick((9, 1), 0, None), tick((10, 1), 0, None)])
        self.PartEqual(log(tickpreexps=[log.pre1exp, log.pre1to9exp], labelpreexps=[]).partfunctions(1, 10, 1, 1)[0](),
                       [tick((1, 1), 0, None), tick((2, 1), 1, None), tick((3, 1), 1, None), tick((4, 1), 1, None), tick((5, 1), 1, None), tick((6, 1), 1, None), tick((7, 1), 1, None), tick((8, 1), 1, None), tick((9, 1), 1, None), tick((10, 1), 0, None)])
        self.PartEqual(log(tickpreexps=[log.pre1exp, log.pre1to9exp]).partfunctions(1, 10, 1, 1)[0](),
                       [tick((1, 1), 0, 0), tick((2, 1), 1, None), tick((3, 1), 1, None), tick((4, 1), 1, None), tick((5, 1), 1, None), tick((6, 1), 1, None), tick((7, 1), 1, None), tick((8, 1), 1, None), tick((9, 1), 1, None), tick((10, 1), 0, 0)])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import math
from pyx import *

def drawexample(canvas, corner, linealign):
    r = 1.5
    if corner:
      canvas.stroke(path.path(path.arc(5, 5, r, 0, 360)))
    else:
      canvas.stroke(path.path(path.arc(5.5, 5+math.sqrt(3)/6, r, 0, 360)))
    phi = 0
    while phi < 2 * math.pi + 1e-10:
        if corner:
            b = box.polygon(center=(5, 5), corners=((5, 5), (6, 5), (5.5, 5+math.sqrt(3)/2)))
        else:
            b = box.polygon(center=(5.5, 5+math.sqrt(3)/6), corners=((5, 5), (6, 5), (5.5, 5+math.sqrt(3)/2)))
        if linealign:
            b.linealign(r, math.cos(phi), math.sin(phi))
        else:
            b.circlealign(r, math.cos(phi), math.sin(phi))
        if round(phi / math.pi * 2 * 100) % 100:
            canvas.stroke(b.path(centerradius=0.05))
        else:
            canvas.stroke(b.path(centerradius=0.05), [color.rgb.red])
        phi += math.pi / 50

c = canvas.canvas()
sc = c.insert(canvas.canvas([trafo.translate(0, 6)]))
drawexample(sc, 0, 0)
sc = c.insert(canvas.canvas([trafo.translate(6, 6)]))
drawexample(sc, 0, 1)
sc = c.insert(canvas.canvas([trafo.translate(0, 0)]))
drawexample(sc, 1, 0)
sc = c.insert(canvas.canvas([trafo.translate(6, 0)]))
drawexample(sc, 1, 1)
c.writeEPSfile("test_box", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_box", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_box", page_paperformat=document.paperformat.A4)




# You can manually set the subaxes and exceed the axes ranges.
# The width of the regular axes can be modified by a size parameter.
# While most axes do not have a size parameter, it can be added to
# any existing axis very easily. For linear axes so called sizedlinear
# and autosizedlinear axes are defined by PyX already.

from pyx import *

subaxes = [graph.axis.linear(max=1),
           graph.axis.linear(max=1),
           graph.axis.sizedlinear(size=3, min=0, max=3)]

g = graph.graphxy(width=8, y=graph.axis.split(subaxes=subaxes, epsilon=None))
g.plot([graph.data.file("shift.dat", x=1, y="i, $(i+2)", context={"i": i})
        for i in range(3)], [graph.style.line()])
g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



from pyx import *

text.preamble(r"\parindent0pt")

c = canvas.canvas()
t = c.text(0, 0, r"spam \& eggs", [trafo.scale(6), text.parbox(1.2, baseline=text.parbox.top)])
t2 = text.text(0, 0, "eggs", [trafo.scale(6)])
b, b2 = t.bbox(), t2.bbox()
c.stroke(t.path(), [style.linewidth.THin])
c.stroke(path.line(-0.3, b.top(), -0.1, b.top()), [deco.earrow.Small])
c.text(-0.5, b.top(), "valign.top", [text.vshift.mathaxis, text.halign.right])
c.stroke(path.line(-0.3, 0.5*(b.top()+b.bottom()), -0.1, 0.5*(b.top()+b.bottom())), [deco.earrow.Small])
c.text(-0.5, 0.5*(b.top()+b.bottom()), "valign.middle", [text.vshift.mathaxis, text.halign.right])
c.stroke(path.line(-0.3, b.bottom(), -0.1, b.bottom()), [deco.earrow.Small])
c.text(-0.5, b.bottom(), "valign.bottom", [text.vshift.mathaxis, text.halign.right])
c.stroke(path.line(0, 0, 7.2, 0))
c.stroke(path.line(7.3, 0, 7.5, 0), [deco.barrow.Small])
c.text(7.7, 0, "parbox.top", [text.vshift.mathaxis])
c.stroke(path.line(7.3, 0.5*(b.bottom()-b2.bottom()), 7.5, 0.5*(b.bottom()-b2.bottom())), [deco.barrow.Small])
c.text(7.7, 0.5*(b.bottom()-b2.bottom()), "parbox.middle", [text.vshift.mathaxis])
c.stroke(path.line(0, b.bottom()-b2.bottom(), 7.2, b.bottom()-b2.bottom()))
c.stroke(path.line(7.3, b.bottom()-b2.bottom(), 7.5, b.bottom()-b2.bottom()), [deco.barrow.Small])
c.text(7.7, b.bottom()-b2.bottom(), "parbox.bottom", [text.vshift.mathaxis])
c.writePDFfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *
from pyx.connector import *


startbox = box.polygon(corners=[[0,0], [1,0.2], [0.5,2]])
endbox = box.polygon(corners=[[10,3.8], [13,5.2], [12.5,4.0]])

def dotest(c, x, y, test):
   c2 = c.insert(canvas.canvas([trafo.translate(x, y)]))
   c2.stroke(startbox.path())
   c2.stroke(endbox.path())
   c2.stroke(path.circle( unit.pt * startbox.center[0], unit.pt * startbox.center[1], 0.1))
   c2.stroke(path.circle( unit.pt * endbox.center[0], unit.pt * endbox.center[1], 0.1))
   eval("%s(c2)" % test)
   c.insert(c2)


def testline(c):
    l = line(startbox, endbox)
    c.stroke(l, [style.linewidth.THICK, color.rgb.red, deco.earrow.normal])

    l = line(endbox, startbox, boxdists=[1,0])
    c.stroke(l, [color.rgb.blue, deco.earrow.normal])


def testarc(c):
    l = arc(endbox, startbox, boxdists=[1,0])
    c.stroke(l, [color.rgb.blue, deco.earrow.normal])

    for a in [-135, -90, 0, 45, 90]:
        l = arc(startbox, endbox, relangle=a, boxdists=[0.5,0.5])
        c.stroke(l, [color.rgb.red, deco.earrow.normal])

    for b in [-1.1, -0.4, 0, 0.5]:
        l = arc(startbox, endbox, relbulge=b, boxdists=[1,1])
        c.stroke(l, [color.rgb.green, deco.earrow.normal])

    l = arc(startbox, endbox, absbulge=5, relbulge=0, boxdists=[1,0])
    c.stroke(l, [color.rgb.blue, deco.earrow.normal])


def testcurve(c):
    l = curve(endbox, startbox, boxdists=[1,0])
    c.stroke(l, [color.rgb.blue, deco.earrow.normal])

    for a in [ [ 90,  90,  1.0], \
               [-90,  90,  1.0], \
               [ 90, -90,  1.0], \
               [-90,   0,  1.0], \
               [ 20,  20,  2.0] ]:
        l = curve(startbox, endbox, boxdists=[0.8,0.8],
            relangle1=a[0], relangle2=a[1], relbulge=a[2])
        c.stroke(l, [color.rgb.red, deco.earrow.normal])

    for a in [ [ 90,  90,  1.0], \
               [-90,  90,  1.0] ]:
        l = curve(startbox, endbox, boxdists=[0.8,0.8],
            absangle1=a[0], absangle2=a[1], relbulge=a[2])
        c.stroke(l, [color.rgb.green, deco.earrow.normal])


def testtwolines(c):
    l = twolines(endbox, startbox, relangle1=45, relangle2=45)
    c.stroke(l, [color.rgb.blue, deco.earrow.normal])

    for a in [ [0,90], [90,0] ]:
        l = twolines(startbox, endbox, boxdists=[0.8,0.8],
            absangle1=a[0], absangle2=a[1])
        c.stroke(l, [color.rgb.red, deco.earrow.normal])

    for a in range(10,20,2):
        l = twolines(startbox, endbox, boxdists=[0.8,0.8],
            absangle1=45, length2=a)
        c.stroke(l, [color.rgb.green, deco.earrow.normal])

    for a in range(5,20,2):
        l = twolines(startbox, endbox, boxdists=[0.8,0.8],
            relangle1=45, length1=a)
        c.stroke(l, [color.rgb.green, deco.earrow.normal])


c = canvas.canvas()
dotest(c,  0, 0, "testline")
dotest(c, 25, 0, "testarc")
dotest(c, 25,30, "testcurve")
dotest(c,  0,20, "testtwolines")
c.writeEPSfile("test_connector", page_paperformat=document.paperformat.A4, page_rotated=0, page_fittosize=1)
c.writePDFfile("test_connector", page_paperformat=document.paperformat.A4, page_rotated=0, page_fittosize=1)
c.writeSVGfile("test_connector", page_paperformat=document.paperformat.A4, page_rotated=0, page_fittosize=1)




from math import pi
from pyx import *
from pyx.graph import axis

g = graph.graphxy(width=8, key=graph.key.key(pos="bl"),
        x=axis.linear(min=0, max=2*pi, title="$x$", divisor=pi,
                           texter=axis.texter.rational(suffix=r"\pi")),
        y=axis.linear(title="$y$"))

g.plot(graph.data.function("y(x)=sin(x)", title=r"$\sin(x)$"))
g.plot(graph.data.function("y(x)=cos(x)", title=r"$\cos(x)$"))

g.finish()
g.stroke(g.ygridpath(0))

g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()
